/*
 * XYZ Library - clip-origin.c
 *
 * Copyright (C) 2001 Woodley Packard
 * 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include	<X11/X.h>
#include	<X11/Xlib.h>
#include	<X11/Xutil.h>
#include	<X11/keysymdef.h>
#include	<X11/Xos.h>

#include	<yz/window.h>
#include	<yz/draw.h>
#include	"window_data.h"

#include	<stdlib.h>
#include	<stdio.h>
#include	<time.h>

extern Display		*display;
extern Colormap		cmap;
extern int			screen;

extern window_t	*yz_current_window;
extern window_data	*yz_d;

int	yzSetClipRect2i2i(int	x1, int	y1, int	x2, int	y2)
{
	transform_point(&x1, &y1); transform_point(&x2, &y2);
	return do_set_clip_rect(x1, y1, x2, y2);
}

int	do_set_clip_rect(int	x1, int	y1, int	x2, int	y2)
{
	XRectangle	rect;

	if(!yz_current_window)return -1;
	//printf("clip now %d, %d, %d, %d\n", x1, y1, x2, y2);
	rect.x = x1;
	rect.y = y1;
	rect.width = x2-x1+1;
	rect.height = y2-y1+1;
	XSetClipRectangles(display, yz_d->gc, 0, 0, &rect, 1, YXSorted);
	XSetClipRectangles(display, yz_d->pgc, 0, 0, &rect, 1, YXSorted);
	return 0;
}

int	yzPushClipRect2i2i(int	x1, int	y1, int	x2, int	y2)
{
	window_data	*d;
	rect_info	or, ar, *nr;
	
	if(!yz_current_window)return -1;
	d = yz_current_window->impl_data;

	transform_point(&x1, &y1); transform_point(&x2, &y2);

	// first make sure we have room on the stack
	if((d->cs.n+1) >= d->cs.a)
	{
		d->cs.a = (d->cs.a + 2) * 2;
		nr = realloc(d->cs.r, sizeof(rect_info) * d->cs.a);
		if(!nr)return -1;
		else d->cs.r = nr;
	}

	// clip this rect to the old clipping rect if necessary
	if(d->cs.n)
	{
		or = d->cs.r[d->cs.n-1];
		if(or.x1 > x1)x1 = or.x1;
		if(or.y1 > y1)y1 = or.y1;
		if(or.x2 < x2)x2 = or.x2;
		if(or.y2 < y2)y2 = or.y2;
	}

	// save the new rect and activate it
	nr = &d->cs.r[d->cs.n];
	d->cs.n++;
	nr->x1 = x1; nr->y1 = y1;
	nr->x2 = x2; nr->y2 = y2;
	return do_set_clip_rect(x1, y1, x2, y2);
}

int yzPopClipRect()
{
    return yzPopClipRect4p(0,0,0,0);
}

int yzPopClipRect4p(int *X1, int    *Y1, int    *X2, int    *Y2)
{
    int ox1, oy1, ox2, oy2;
    int nx1, ny1, nx2, ny2;
    window_data *d;

    if(!yz_current_window)return -1;
    d = yz_current_window->impl_data;
    if(d->cs.n<=0)
    {
        fprintf(stderr, "warning: clip stack underflow\n");
        return -1;
    }
    d->cs.n--;
    ox1 = d->cs.r[d->cs.n].x1;
    oy1 = d->cs.r[d->cs.n].y1;
    ox2 = d->cs.r[d->cs.n].x2;
    oy2 = d->cs.r[d->cs.n].y2;

    if(d->cs.n) // there's an item beneath this
        do_set_clip_rect(   nx1 = d->cs.r[d->cs.n-1].x1,
                            ny1 = d->cs.r[d->cs.n-1].y1,
                            nx2 = d->cs.r[d->cs.n-1].x2,
                            ny2 = d->cs.r[d->cs.n-1].y2 );
    else    // popping the last item in the stack
        do_set_clip_rect(   nx1 = 0, ny1 = 0, nx2 = (yz_current_window->width-1),
                    ny2 = (yz_current_window->height-1) );

    if(X1)*X1 = ox1 - yz_current_window->origin_x;
    if(Y1)*Y1 = oy1 - yz_current_window->origin_y;
    if(X2)*X2 = ox2 - yz_current_window->origin_x;
    if(Y2)*Y2 = oy2 - yz_current_window->origin_y;

    return 0;
}

int	extend_clip_on_resize()
{
    int nx1, ny1, nx2, ny2;
    window_data *d;

    if(!yz_current_window)return -1;
    d = yz_current_window->impl_data;
    if(!d->cs.n)
	{
		do_set_clip_rect(   nx1 = 0, ny1 = 0, nx2 = (yz_current_window->width-1),
			               ny2 = (yz_current_window->height-1) );
	}
}

/*
int	yzPopClipRect()
{
	window_data	*d;
	
	if(!yz_current_window)return -1;
	d = yz_current_window->impl_data;
	if(d->cs.n<=0)
	{
		fprintf(stderr, "warning: clip stack underflow\n");
		return -1;
	}
	d->cs.n--;
	if(d->cs.n)	// there's an item beneath this
		do_set_clip_rect(	d->cs.r[d->cs.n-1].x1,
					d->cs.r[d->cs.n-1].y1,
					d->cs.r[d->cs.n-1].x2,
					d->cs.r[d->cs.n-1].y2	);
	else	// popping the last item in the stack
		do_set_clip_rect(	0, 0, yz_current_window->width-1,
					yz_current_window->height-1	);
	return 0;
}
*/

int	yzSetOrigin(int	ox, int	oy)
{
	if(!yz_current_window)return -1;
	yz_current_window->origin_x = ox;
	yz_current_window->origin_y = oy;
	return 0;
}

int	yzOffsetOrigin(int	ox, int	oy)
{
	window_data	*d;
	point_info	*np;
	
	if(!yz_current_window)return -1;
	d = yz_current_window->impl_data;
	yz_current_window->origin_x += ox;
	yz_current_window->origin_y += oy;
	if(d->os.n)
	{
		np = &d->os.p[d->os.n-1];
		np->x = yz_current_window->origin_x;
		np->y = yz_current_window->origin_y;
	}
	return 0;
}

int	yzPushOrigin(int	x, int	y)
{
	window_data	*d;
	point_info	*np;
	
	if(!yz_current_window)return -1;
	d = yz_current_window->impl_data;

	transform_point(&x, &y);

	// first make sure we have room on the stack
	if((d->os.n+1) >= d->os.a)
	{
		d->os.a = (d->os.a + 2) * 2;
		np = realloc(d->os.p, sizeof(point_info) * d->os.a);
		if(!np)return -1;
		else d->os.p = np;
	}

	// save the new origin and activate it
	np = &d->os.p[d->os.n];
	d->os.n++;
	np->x = x; np->y = y;
	//fprintf(stderr, "debug: push origin %d %d\n", x, y);
	return yzSetOrigin(x, y);
}

int	yzPopOrigin()
{
	return yzPopOrigin2p(0,0);
}

int yzPopOrigin2p(int   *X, int *Y)
{
	int		 oox, ooy, nox, noy;
	window_data *d;

	if(!yz_current_window)return -1;
	d = yz_current_window->impl_data;
	if(d->os.n<=0)
	{
		fprintf(stderr, "warning: origin stack underflow\n");
		return -1;
	}
	d->os.n--;
	oox = d->os.p[d->os.n].x;   // remember
	ooy = d->os.p[d->os.n].y;

	if(d->os.n) // there's an item beneath this
		yzSetOrigin(		nox = d->os.p[d->os.n-1].x,
					noy = d->os.p[d->os.n-1].y);
	else	// popping the last item in the stack
		yzSetOrigin(nox = 0, noy = 0);

	if(X)*X = oox - yz_current_window->origin_x;
	if(Y)*Y = ooy - yz_current_window->origin_y;
	//fprintf(stderr, "debug: pop origin\n");

	return 0;
}

/*
int	yzPopOrigin()
{
	window_data	*d;
	
	if(!yz_current_window)return -1;
	d = yz_current_window->impl_data;
	if(d->os.n<=0)
	{
		fprintf(stderr, "warning: origin stack underflow\n");
		return -1;
	}
	d->os.n--;
	if(d->os.n)	// there's an item beneath this
		yzSetOrigin(		d->os.p[d->os.n-1].x,
					d->os.p[d->os.n-1].y);
	else	// popping the last item in the stack
		yzSetOrigin(0, 0);
	return 0;
}
*/
