#ifdef USE_PANGO

#define	_GNU_SOURCE
#include	<string.h>

#include    <pango/pango.h>
#include    <pango/pangoxft.h>
#include    <yz/window.h>
#include    "window_data.h"

#define     PANGO_UNITS(d)  (d<<10)

#define     DEFAULT_FONT    "Sans 10"

PangoContext            *pango_ctx;
XftDraw                 *xftdraw;
PangoFontDescription    *default_font;
FILE                    *logfile;


extern Display      *display;
extern int          screen;
extern Colormap     cmap;
extern window_t     *yz_current_window;
extern window_data  *yz_d;

font_t                  *no_window_font = 0;
PangoFontDescription    *no_window_x11_font = 0;

int pango_render_text(Drawable drw, char *str, int n, int x, int y, GC gc) {
    if (!pango_ctx) {
        return -1;
    }

    int xx = x;
    int yy = y;

    x = PANGO_UNITS(x);
    /* add 4 pixel to y: 
     * no idea why but otherwise the text appears off place.. */
    y = PANGO_UNITS(y);
    //y = PANGO_UNITS(y+4);
    

    /* convert GC colour to XftColor */
    XGCValues gcvalues;
    XColor dummyc;
    XftColor xftcolor;
    XGetGCValues(display, gc, GCForeground | GCBackground, &gcvalues);
    dummyc.pixel = gcvalues.foreground;
    XQueryColor(display, cmap, &dummyc);
    xftcolor.color.red = dummyc.red;
    xftcolor.color.green = dummyc.green;
    xftcolor.color.blue = dummyc.blue;
    xftcolor.color.alpha =  0xffff;
    xftcolor.pixel = gcvalues.foreground;
    

    PangoLayout *layout = pango_layout_new(pango_ctx);

    /* set text */
    pango_layout_set_text(layout, str, n);
    /* set font */
    pango_layout_set_font_description(layout, yz_d->fnt);
  
    /* different co-ordinate system, flip y */
    int w, h;
    pango_layout_get_size(layout, &w, &h);
    //y -= h;

    PangoRectangle ink_size, inkk_size, ink_pango;
    pango_layout_get_pixel_extents(layout, &inkk_size, &ink_size);
    PangoLayoutIter *layout_iter = pango_layout_get_iter(layout);
    yy -= PANGO_PIXELS(pango_layout_iter_get_baseline(layout_iter));
    pango_layout_iter_free(layout_iter);
    //yy -= inkk_size.y + inkk_size.height;
    pango_layout_get_extents(layout, NULL, &ink_pango);
    //y += ink_pango.height;
    FILE *f=stderr;
    //f=fopen("/tmp/log", "a");
    fprintf(f,"str %s, font %s, pos: %d, %d\n", str, pango_font_description_to_string(yz_d->fnt), xx, yy);
    fprintf(f,"ink size: %d, %d, %d, %d\n", inkk_size.x, inkk_size.y, inkk_size.width, inkk_size.height);
    fprintf(f,"logical size: %d, %d, %d, %d\n", ink_size.x, ink_size.y, ink_size.width, ink_size.height);
    //fclose(f);

    Pixmap pixmap = XCreatePixmap(display, drw, ink_size.width, ink_size.height, DefaultDepth(display, screen));
    
    XGCValues ngcvalues;
    ngcvalues.foreground = ngcvalues.background = 0xFFFFFF;
    GC ngc = XCreateGC(display, pixmap, GCForeground | GCBackground, &ngcvalues); 

    XFillRectangle(display, pixmap, ngc, 0, 0, ink_size.width, ink_size.height);
    XFreeGC(display, ngc);
    //XDrawRectangle(display, pixmap, gc, 0, 0, ink_size.width, ink_size.height);
    //XDrawLine(display, pixmap, drw, 0, 0, 5, 5);
    XftDraw *xdrw = XftDrawCreate(display, pixmap, DefaultVisual(display, screen), cmap);
    //XftDraw *xdrw = XftDrawCreateAlpha(display, pixmap, DefaultDepth(display, screen));
    /* render the text */
    /*pango_xft_render_layout(
            XftDrawCreate(display, drw, DefaultVisual(display, screen), cmap),
            &xftcolor, layout, x, y
    );*/
    XCopyArea(display, drw, pixmap, gc, xx, yy, ink_size.width, ink_size.height, 0, 0);
    //XCopyArea(display, drw, pixmap, gc, xx+ink_size.x, yy+ink_size.y, ink_size.width, ink_size.height, 0, 0);
    pango_xft_render_layout(xdrw, &xftcolor, layout, 0, 0);//PANGO_UNITS(5));
    XftDrawDestroy(xdrw);
    XCopyArea(display, pixmap, drw, gc, ink_size.x, ink_size.y, ink_size.width, ink_size.height, xx+ink_size.x, yy+ink_size.y);
    XFreePixmap(display, pixmap);

    return 0;
}


font_t *alloc_font() {
    font_t *f;
    
    f = malloc(sizeof(font_t));
    f->impl_data = malloc(sizeof(struct font_data));
    ((font_data*)f->impl_data)->refcnt = 1;
    return f;
}

free_font(font_t *fnt) {
    font_data   *fi;

    fi = (font_data*)fnt->impl_data;
    fi->refcnt--;
    if (fi->refcnt>0)
        return 0;

    free(fnt->impl_data);
    free(fnt);
}

font_t *font_from_name(const char *font_name) {
    font_t  *f;
    char *c;
    f = alloc_font();
    if (c = index(font_name, '-')) {
        *c = ' ';
        strncpy(f->name, font_name, c - font_name);
        f->name[c - font_name] = 0;
        char *attr = NULL;
        if (attr = strcasestr(c, "bi")) {
            strcat(f->name, " Bold Oblique");
        } else if (attr = strcasestr(c, "b")) {
            strcat(f->name, " Bold");
        } else if (attr = strcasestr(c, "i")) {
            strcat(f->name, " Oblique");
        }
        if (attr) {
            strncat(f->name, c, attr-c);
        } else {
            strcat(f->name, c);
        }
    } else {
        strcpy(f->name, font_name);
    }

    if (strcasecmp(font_name, "system") != 0) {
        ((font_data*)f->impl_data)->fd = pango_font_description_from_string(f->name);
    } else {
        ((font_data*)f->impl_data)->fd = pango_font_description_copy_static(default_font);
    }

    return f;
}

font_t *get_system_font() {
    return font_from_name(DEFAULT_FONT);
}

int do_select_font() {
}

yzSelectFont1s(char *font_name)
{
    font_t  *nf;

    nf = font_from_name(font_name);
    if (!nf)
        return -1;
    yzSelectFont1f(nf);
    return 0;
}

yzSelectFont1f(font_t *nf)
{
    font_data *fd;
    font_t *of = 0;

    fd = nf->impl_data;
    fd->refcnt++;

    if(!yz_current_window || yz_d->is_postscript) {
        of = no_window_font;
        no_window_font = nf;
        no_window_x11_font = fd->fd;
    } else {
        of = yz_current_window->font;
        yz_current_window->font = nf;
        yz_d->fnt = fd->fd;
    }
    if (of)
        free_font(of);
    return 0;
}

int non_window_string_length(char *s, int n) {
    if (!display)
        init_xyz();
    if (no_window_font)
        return font_string_width(no_window_x11_font, s, n);
    else
        return -1;
}

font_t *yzGetFont1s(char *font_name) {
    font_t *f = font_from_name(font_name);
    return f;
}

int yzFreeFont(font_t *f) {
    free_font(f);
    return 0;
}

int font_string_width(PangoFontDescription *fnt, char *s, int n) {
    int w, h;
    PangoRectangle prect;
    PangoLayout *layout = pango_layout_new(pango_ctx);
    pango_layout_set_text(layout, s, n);
    pango_layout_set_font_description(layout, fnt);
    //pango_layout_get_size(layout, &w, &h);
    pango_layout_get_pixel_extents(layout, NULL, &prect);
    return prect.width;
    //return PANGO_PIXELS(w);
}

int	default_line_height(PangoFontDescription	*fnt_d)
{
	if(!display)init_xyz();
	PangoFontMetrics	*met = pango_context_get_metrics(pango_ctx, fnt_d, NULL);

	int	ascent = pango_font_metrics_get_ascent(met);
	int	descent = pango_font_metrics_get_descent(met);
	int	spacing = (ascent + descent) * 1.0;
	//printf("PANGO ascent %d descent %d suggesting %d spacing\n", ascent, descent, spacing);
	pango_font_metrics_unref(met);
	return spacing>>10;
}

int pango_init() {
    pango_ctx = pango_xft_get_context(display, 0);
    default_font = pango_font_description_from_string(DEFAULT_FONT);
    no_window_font = font_from_name(DEFAULT_FONT);
    no_window_x11_font  = ((font_data*)no_window_font->impl_data)->fd;
}



#endif
